param(
    [Parameter(Mandatory = $true)]
    [string]$path_output_lehrer,
    [Parameter(Mandatory = $true)]
    [string]$path_output_schueler,
    [Parameter(Mandatory = $true)]
    [string]$security_group_lehrer_name,
    [Parameter(Mandatory = $true)]
    [string]$security_group_schueler_name,
    [Parameter(Mandatory = $true)]
    [string]$schueler_list_path,
    [Parameter(Mandatory = $true)]
    [string]$lehrer_list_path
)

if (!(Test-Path -Path $schueler_list_path -PathType leaf)) {
    throw "Pfad zur Schuelerliste existert nicht" 
}
if (!(Test-Path -Path $lehrer_list_path -PathType leaf)) {
    throw "Pfad zur Lehrerliste existert nicht" 
}

Install-Module -Name AzureAD

Connect-AzureAD

function Get-RandomCharacters($length, $characters) { 
    $random = 1..$length | ForEach-Object { Get-Random -Maximum $characters.length } 
    $private:ofs = "" 
    return [String]$characters[$random]
}

            
$schueler_list = Import-Csv -Path $schueler_list_path -Delimiter ";"
$lehrer_list = Import-Csv -Path $lehrer_list_path -Delimiter ";"

$licence_schueler = "STANDARDWOFFPACK_STUDENT"
$licence_lehrer = "STANDARDWOFFPACK_FACULTY"

$security_group_lehrer = Get-AzureADGroup | Where-Object Displayname -eq $security_group_lehrer_name
$security_group_schueler = Get-AzureADGroup | Where-Object Displayname -eq $security_group_schueler_name

#security Gruppen erstellen falls noch nicht vorhanden
if ($security_group_lehrer.count -lt 1) {
    $security_group_lehrer = New-AzureADGroup -DisplayName $security_group_lehrer_name -MailEnabled $false -SecurityEnabled $true -MailNickName "notSet"
}
if ($security_group_schueler.count -lt 1) {
    $security_group_schueler = New-AzureADGroup -DisplayName $security_group_schueler_name -MailEnabled $false -SecurityEnabled $true -MailNickName "notSet"
}

$created_schueler = @()
$created_lehrer = @()

foreach ($schueler in $schueler_list) {

    try {

        # zufälliges Kennwort wird generiert 
        $password = Get-RandomCharacters -length 4 -characters "ABCDEFHIJKLMOPQRSTUVXYZ123456789"
        $password += Get-RandomCharacters -length 4 -characters "abcdefghijklmopqrstuvxyz123456789"
        $password += Get-RandomCharacters -length 1 -characters "123456789"
        $password += Get-RandomCharacters -length 1 -characters "!?"
        
        $PasswordProfile = New-Object -TypeName Microsoft.Open.AzureAD.Model.PasswordProfile
        $PasswordProfile.Password = $password
        $PasswordProfile.ForceChangePasswordNextLogin = $true

        $user = New-AzureADUser -DisplayName $schueler.DisplayName -GivenName $schueler.GivenName -SurName $schueler.SurName -UserPrincipalName $schueler.UPN `
            -UsageLocation $schueler.Location -MailNickName $schueler.Mail -PasswordProfile $PasswordProfile -AccountEnabled $true

        #zur Gruppe hinzufügen
        Add-AzureADGroupMember -ObjectId $security_group_schueler.ObjectId -RefObjectId $user.ObjectId

        $newUser = New-Object -TypeName psobject 
        $newUser | Add-Member -MemberType NoteProperty -Name "DisplayName" -Value $user.DisplayName
        $newUser | Add-Member -MemberType NoteProperty -Name "UPN" -Value $user.UserPrincipalName
        $newUser | Add-Member -MemberType NoteProperty -Name "Passwort" -Value $password

        $created_schueler += $newUser

        $License = New-Object -TypeName Microsoft.Open.AzureAD.Model.AssignedLicense
        $License.SkuId = (Get-AzureADSubscribedSku | Where-Object -Property SkuPartNumber -Value $licence_schueler -EQ).SkuID
        $LicensesToAssign = New-Object -TypeName Microsoft.Open.AzureAD.Model.AssignedLicenses
        $LicensesToAssign.AddLicenses = $License

        Set-AzureADUserLicense -ObjectId $user.UserPrincipalName -AssignedLicenses $LicensesToAssign


        Clear-Variable user
        Clear-Variable newUser
    }
    catch {
        write-host "Fehler beim erstellen von einem User: "
        write-host "Betroffener User: "
        write-host $schueler.DisplayName
        write-host $_.Exception.Message
        write-host "---------------"
    }
         
} 
         
foreach ($lehrer in $lehrer_list) {

    try {
        # zufälliges Kennwort wird generiert 
        $password = Get-RandomCharacters -length 4 -characters "ABCDEFHIJKLMOPQRSTUVXYZ123456789"
        $password += Get-RandomCharacters -length 4 -characters "abcdefghijklmopqrstuvxyz123456789"
        $password += Get-RandomCharacters -length 1 -characters "123456789"
        $password += Get-RandomCharacters -length 1 -characters "!?"

        $PasswordProfile = New-Object -TypeName Microsoft.Open.AzureAD.Model.PasswordProfile
        $PasswordProfile.Password = $password
        $PasswordProfile.ForceChangePasswordNextLogin = $true

        $user = New-AzureADUser -DisplayName $lehrer.DisplayName -GivenName $lehrer.GivenName -SurName $lehrer.SurName -UserPrincipalName $lehrer.UPN `
            -UsageLocation $lehrer.Location -MailNickName $lehrer.Mail -PasswordProfile $PasswordProfile -AccountEnabled $true

        #zur Gruppe hinzufügen
        Add-AzureADGroupMember -ObjectId $security_group_lehrer.ObjectId -RefObjectId $user.ObjectId

        $newUser = New-Object -TypeName psobject 
        $newUser | Add-Member -MemberType NoteProperty -Name "DisplayName" -Value $user.DisplayName
        $newUser | Add-Member -MemberType NoteProperty -Name "UPN" -Value $user.UserPrincipalName
        $newUser | Add-Member -MemberType NoteProperty -Name "Passwort" -Value $password

        $created_lehrer += $newUser

        $License = New-Object -TypeName Microsoft.Open.AzureAD.Model.AssignedLicense
        $License.SkuId = (Get-AzureADSubscribedSku | Where-Object -Property SkuPartNumber -Value $licence_lehrer -EQ).SkuID
        $LicensesToAssign = New-Object -TypeName Microsoft.Open.AzureAD.Model.AssignedLicenses
        $LicensesToAssign.AddLicenses = $License
        Set-AzureADUserLicense -ObjectId $user.UserPrincipalName -AssignedLicenses $LicensesToAssign

        Clear-Variable user
        Clear-Variable newUser
    }
    catch {
        write-host "Fehler beim erstellen von einem User: "
        write-host "Betroffener User: "
        write-host $schueler.DisplayName
        write-host $_.Exception.Message
        write-host "---------------"
                
    }
}

$created_schueler | Export-Csv -Path $path_output_schueler -NoTypeInformation
$created_lehrer | Export-Csv -Path $path_output_lehrer -NoTypeInformation

Write-Host "Benutzer erfolgreich hinzugefuegt und Lizenzen zugewiesen"
